﻿using Hotline.KnowledgeBase;
using Hotline.Repository.SqlSugar.Extensions;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.Knowledge;
using Hotline.Share.Enums.KnowledgeBase;
using MapsterMapper;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using SqlSugar;
using System.Xml.Linq;
using XF.Domain.Authentications;
using XF.Domain.Exceptions;

namespace Hotline.Api.Controllers
{
    /// <summary>
    /// 
    /// </summary>
    public class KnowledgeController : BaseController
    {
        private readonly IKnowledgeRepository _knowledgeRepository;
        private readonly IKnowledgeTempRepository _knowledgeTempRepository;
        private readonly IKnowledgePVRepository _knowledgePVRepository;
        private readonly ISessionContext _sessionContext;
        private readonly IKnowledgeService _knowledgeService;
        private readonly IMapper _mapper;

        /// <summary>
        /// 登录信息
        /// </summary>
        /// <param name="knowledgeRepository"></param>
        /// <param name="knowledgeTempRepository"></param>
        /// <param name="knowledgePVRepository"></param>
        /// <param name="sessionContext"></param>
        /// <param name="knowledgeService"></param>
        /// <param name=""></param>
        /// <param name="mapper"></param>
        public KnowledgeController(IKnowledgeRepository knowledgeRepository, IKnowledgeTempRepository knowledgeTempRepository,
            IKnowledgePVRepository knowledgePVRepository, ISessionContext sessionContext, IKnowledgeService knowledgeService
            , IMapper mapper)
        {
            _knowledgeRepository = knowledgeRepository;
            _knowledgeTempRepository = knowledgeTempRepository;
            _knowledgePVRepository = knowledgePVRepository;
            _sessionContext = sessionContext;
            _knowledgeService = knowledgeService;
            _mapper = mapper;
        }
        #region 草稿箱
        /// <summary>
        /// 草稿箱-新增
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("knowledgetemp-add")]
        public async Task<string> KnowledgeTempAdd([FromBody] AddKnowledgeDto dto)
        {
            var kn = _mapper.Map<KnowledgeTemp>(dto);
            if (string.IsNullOrEmpty(dto.Code))
                kn.Code = Convert.ToInt64((DateTime.Now - new DateTime(1970, 1, 1, 0, 0, 0, 0)).TotalSeconds).ToString();
            kn.PageView = 0;
            kn.CreationTime = DateTime.Now;
            return await _knowledgeTempRepository.AddAsync(kn);
        }

        /// <summary>
        /// 草稿箱-修改
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("knowledgetemp-update")]
        public async Task KnowledgeTempUpdate([FromBody] UpdateKnowledgeDto dto)
        {
            var kn = _mapper.Map<KnowledgeTemp>(dto);
            await _knowledgeTempRepository.UpdateAsync(kn);
        }

        /// <summary>
        /// 草稿箱-查询详情
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [HttpGet("knowledgetemp-info")]
        public async Task<KnowledgeTemp> KnowledgeTempInfo(string Id)
        {
            var temp = await _knowledgeTempRepository.GetAsync(Id);
            if (temp is null)
                throw UserFriendlyException.SameMessage("知识草稿查询失败！");
            return temp;
        }

        /// <summary>
        /// 草稿箱删除
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [HttpGet("knowledgetemp-delete")]
        public async Task KnowledgeTempDelete(string Id)
        {
            if (string.IsNullOrEmpty(Id))
                throw UserFriendlyException.SameMessage("Id不能为空！");

            await _knowledgeTempRepository.RemoveAsync(p => p.Id == Id);
        }

        #endregion

        /// <summary>
        /// 知识库-知识修改-查询详情
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [HttpGet("knowledge-updateinfo")]
        [AllowAnonymous]
        public async Task<Knowledge> KnowledgeUpdateInfo(string Id)
        {
            var know = await _knowledgeRepository.GetAsync(Id);
            if (know is null)
            {
                //再次查询草稿箱数据
                var tempKnow = await _knowledgeTempRepository.GetAsync(Id);
                if (tempKnow is null)
                    throw UserFriendlyException.SameMessage("知识查询失败！");
                know = new Knowledge();
                _mapper.Map(tempKnow, know);
            }
            return know;
        }

        /// <summary>
        /// 知识库-查询详情-增加浏览量
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [HttpGet("knowledge-info")]
        public async Task<Knowledge> KnowledgeInfo(string Id)
        {
            var know = await _knowledgeRepository.GetAsync(Id);
            if (know is null)
                throw UserFriendlyException.SameMessage("知识查询失败！");
            //写入浏览记录
            KnowledgePv knowledgePv = new()
            {
                CreationTime = DateTime.Now,
                KnowledgeId = know.Id
            };
            await _knowledgePVRepository.AddAsync(knowledgePv);
            return know;
        }

        /// <summary>
        /// 知识库-新增
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-add")]
        public async Task<string> KnowledgeAdd([FromBody] AddKnowledgeDto dto)
        {
            var kn = _mapper.Map<Knowledge>(dto);
            //知识草稿ID如果不为空，则表示是草稿箱的数据
            if (!string.IsNullOrEmpty(dto.Code))
            {
                //移除草稿箱数据
                await _knowledgeTempRepository.RemoveAsync(p => p.Code == dto.Code);
            }
            //如果草稿箱不存在数据，也就是Code为空，从新生成Code
            if (string.IsNullOrEmpty(kn.Code))
                kn.Code = Convert.ToInt64((DateTime.Now - new DateTime(1970, 1, 1, 0, 0, 0, 0)).TotalSeconds).ToString();
            kn.PageView = 0;
            kn.CreationTime = DateTime.Now;
            kn.Status = EKnowledgeStatus.Auditing;

            return await _knowledgeRepository.AddAsync(kn);
        }

        /// <summary>
        /// 知识库-修改
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-update")]
        public async Task KnowledgeUpdate([FromBody] UpdateKnowledgeDto dto)
        {
            //知识修改逻辑
            //如果可以修改，此条数据不动，重新写入一条新数据，知识编号不变，版本号+1

            var know = await _knowledgeRepository.GetAsync(dto.Id);
            if (know is null || know.IsDeleted)
                throw UserFriendlyException.SameMessage("无效知识");
            if (know.Status == EKnowledgeStatus.Auditing || know.Status == EKnowledgeStatus.Update)
                throw UserFriendlyException.SameMessage("审批中数据不能修改");
            var knowledge = _mapper.Map<Knowledge>(dto);
            knowledge.Code = know.Code;
            knowledge.PageView = 0;
            knowledge.CreationTime = DateTime.Now;
            knowledge.Status = EKnowledgeStatus.Auditing;
            knowledge.Id = "";
            knowledge.Version = know.Version + 1;
            //新增一条数据

            await _knowledgeRepository.AddAsync(knowledge);

        }

        /// <summary>
        /// 审批操作
        /// </summary>
        /// <param name="Ids"></param>
        /// <returns></returns>
        [HttpPost("knowledge-approvalhandle")]
        public async Task ApprovalHandle(List<string> Ids)
        {
            if (Ids is null || Ids.Count == 0)
                throw UserFriendlyException.SameMessage("数据不能为空");
            //审批操作
            foreach (var item in Ids)
            {
                var know = await _knowledgeRepository.GetAsync(item);
                if (know != null)
                {
                    //调用流程审批接口

                    //处理数据
                    know.Status = EKnowledgeStatus.Succeed;
                    know.IsOnShelf = true;
                    know.OnShelfTime = DateTime.Now;
                    await _knowledgeRepository.UpdateAsync(know);
                    //处理原始数据的状态
                    var oldKnow = await _knowledgeRepository.GetAsync(p => p.Code == know.Code && p.Status == EKnowledgeStatus.Succeed && p.Id != know.Id);
                    if (oldKnow != null)
                    {
                        oldKnow.Status = EKnowledgeStatus.Update;
                        await _knowledgeRepository.UpdateAsync(oldKnow);
                    }
                }
            }
        }

        #region 知识列表
        /// <summary>
        /// 我的草稿箱
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-mydraftslist")]
        public async Task<PagedDto<KnowledgeDataDto>> MyDraftsList([FromBody] MyDraftsListPagedDto pagedDto)
        {
            var (total, items) = await _knowledgeTempRepository
             .Queryable()
             .Includes(it => it.User)
             .Includes(it => it.KnowledgeType)
             .Where(p => p.CreatorId == _sessionContext.RequiredUserId)
             .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), d => d.Title.Contains(pagedDto.Title!))
             .WhereIF(!string.IsNullOrEmpty(pagedDto.KnowledgeTypeId), d => d.KnowledgeTypeId == pagedDto.KnowledgeTypeId)
             .WhereIF(pagedDto.StartTime != null, d => d.CreationTime >= pagedDto.StartTime)
             .WhereIF(pagedDto.EndTime != null, d => d.CreationTime <= pagedDto.EndTime)
             .OrderByDescending(p => p.CreationTime)
             .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize);

            return new PagedDto<KnowledgeDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(items));
        }

        /// <summary>
        /// 测试列表查询数据
        /// </summary>
        /// <returns></returns>
        [HttpPost("knowledge-testlist")]
        [AllowAnonymous]
        public async Task<string> testlist([FromBody] KnowledgePagedListDto pagedDto)
        {
            var temp = await _knowledgeRepository
               .Queryable().Select(it => new
               {
                   index2 = SqlFunc.RowNumber(it.Code, it.Version),
                   Code = it.Code,
                   Title = it.Title,
                   Version = it.Version
               })
           .MergeTable()//将结果合并成一个表
           .Where(it => it.index2 == 1)

           //.Includes(it => it.User)
           //.Includes(it => it.SystemOrganize)
           //.Includes(it => it.KnowledgeType)
           //.Where(p => p.IsDeleted == false)//&& p.Status != EKnowledgeStatus.Update
           //.WhereIF(!string.IsNullOrEmpty(pagedDto.CreateUserName), d => d.User.Name.Contains(pagedDto.CreateUserName!))
           //.WhereIF(!string.IsNullOrEmpty(pagedDto.CreateBMName), d => d.SystemOrganize.OrgName.Contains(pagedDto.CreateBMName!))
           //.WhereIF(!string.IsNullOrEmpty(pagedDto.Title), d => d.Title.Contains(pagedDto.Title!))
           //.WhereIF(!string.IsNullOrEmpty(pagedDto.KnowledgeTypeId), d => d.KnowledgeTypeId == pagedDto.KnowledgeTypeId)
           //.WhereIF(!string.IsNullOrEmpty(pagedDto.HotspotId), d => d.HotspotId == pagedDto.HotspotId)
           //.WhereIF(pagedDto.StartTime != null, d => d.OnShelfTime >= pagedDto.StartTime)
           //.WhereIF(pagedDto.EndTime != null, d => d.OnShelfTime <= pagedDto.EndTime)

           .ToJsonAsync();
            return temp;
        }

        /// <summary>
        /// 全部知识
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-alllist")]
        public async Task<PagedDto<KnowledgeDataDto>> AllList([FromBody] AllKnowledgePagedListDto pagedDto)
        {
            var bmList = await _knowledgeService.GetChildBMId(_sessionContext.RequiredUserId);
            if (bmList == null || bmList.Count == 0)
                return new PagedDto<KnowledgeDataDto>(0, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(new List<Knowledge>()));
            var (total, items) = await _knowledgeRepository
              .Queryable()
              .Includes(it => it.User)
              .Includes(it => it.SystemOrganize)
              .Includes(it => it.KnowledgeType)
              .Where(p => p.IsDeleted == false && p.Status != EKnowledgeStatus.Update)//
              .Where(p => bmList.Contains(p.CreatorOrgId))//组织Id
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateUserName), d => d.User.Name.Contains(pagedDto.CreateUserName!))//创建人
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateBMName), d => d.SystemOrganize.OrgName.Contains(pagedDto.CreateBMName!))//创建部门
              .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), d => d.Title.Contains(pagedDto.Title!))//知识标题
              .WhereIF(!string.IsNullOrEmpty(pagedDto.KnowledgeTypeId), d => d.KnowledgeTypeId == pagedDto.KnowledgeTypeId)//知识分类
              .WhereIF(!string.IsNullOrEmpty(pagedDto.HotspotId), d => d.HotspotId == pagedDto.HotspotId)//热点
                                                                                                         //创建时间
              .WhereIF(pagedDto.StartTime != null, d => d.CreationTime >= pagedDto.StartTime)
              .WhereIF(pagedDto.EndTime != null, d => d.CreationTime <= pagedDto.EndTime)
               //上架时间
               .WhereIF(pagedDto.StartOnShelfTime != null, d => d.OnShelfTime >= pagedDto.StartTime)
              .WhereIF(pagedDto.EndOnShelfTime != null, d => d.OnShelfTime <= pagedDto.EndTime)
               //下架时间
               .WhereIF(pagedDto.StartOffShelfTime != null, d => d.OffShelfTime >= pagedDto.StartTime)
              .WhereIF(pagedDto.EndOffShelfTime != null, d => d.OffShelfTime <= pagedDto.EndTime)

              .OrderByDescending(p => p.CreationTime)
              .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize);

            return new PagedDto<KnowledgeDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(items));
        }

        /// <summary>
        /// 已上架
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-ontheshelf")]
        public async Task<PagedDto<KnowledgeDataDto>> OnTheShelf([FromBody] KnowledgePagedListDto pagedDto)
        {
            var bmList = await _knowledgeService.GetChildBMId(_sessionContext.RequiredUserId);
            if (bmList == null || bmList.Count == 0)
                return new PagedDto<KnowledgeDataDto>(0, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(new List<Knowledge>()));
            var (total, items) = await _knowledgeRepository
              .Queryable()
              .Includes(it => it.User)
              .Includes(it => it.SystemOrganize)
              .Includes(it => it.KnowledgeType)
              .Where(p => p.IsOnShelf == true && p.Status == EKnowledgeStatus.Succeed && p.IsDeleted == false)
              .Where(p => bmList.Contains(p.CreatorOrgId))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateUserName), d => d.User.Name.Contains(pagedDto.CreateUserName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateBMName), d => d.SystemOrganize.OrgName.Contains(pagedDto.CreateBMName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), d => d.Title.Contains(pagedDto.Title!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.KnowledgeTypeId), d => d.KnowledgeTypeId == pagedDto.KnowledgeTypeId)
              .WhereIF(!string.IsNullOrEmpty(pagedDto.HotspotId), d => d.HotspotId == pagedDto.HotspotId)
              .WhereIF(pagedDto.StartTime != null, d => d.OnShelfTime >= pagedDto.StartTime)
              .WhereIF(pagedDto.EndTime != null, d => d.OnShelfTime <= pagedDto.EndTime)
              .OrderByDescending(p => p.CreationTime)
              .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize);

            return new PagedDto<KnowledgeDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(items));
        }

        /// <summary>
        /// 已下架
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-offshelf")]
        public async Task<PagedDto<KnowledgeDataDto>> OffShelf([FromBody] KnowledgePagedListDto pagedDto)
        {
            var bmList = await _knowledgeService.GetChildBMId(_sessionContext.RequiredUserId);
            if (bmList == null || bmList.Count == 0)
                return new PagedDto<KnowledgeDataDto>(0, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(new List<Knowledge>()));
            var (total, items) = await _knowledgeRepository
              .Queryable()
              .Includes(it => it.User)
              .Includes(it => it.SystemOrganize)
              .Includes(it => it.KnowledgeType)
              .Where(p => p.IsOnShelf == false && p.Status == EKnowledgeStatus.Succeed && p.IsDeleted == false)
              .Where(p => bmList.Contains(p.CreatorOrgId))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateUserName), d => d.User.Name.Contains(pagedDto.CreateUserName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateBMName), d => d.SystemOrganize.OrgName.Contains(pagedDto.CreateBMName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), d => d.Title.Contains(pagedDto.Title!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.KnowledgeTypeId), d => d.KnowledgeTypeId == pagedDto.KnowledgeTypeId)
              .WhereIF(!string.IsNullOrEmpty(pagedDto.HotspotId), d => d.HotspotId == pagedDto.HotspotId)
              .WhereIF(pagedDto.StartTime != null, d => d.OffShelfTime >= pagedDto.StartTime)
              .WhereIF(pagedDto.EndTime != null, d => d.OffShelfTime <= pagedDto.EndTime)
              .OrderByDescending(p => p.CreationTime)
              .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize);

            return new PagedDto<KnowledgeDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(items));
        }

        /// <summary>
        /// 审批中
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-underapproval")]
        public async Task<PagedDto<KnowledgeDataDto>> UnderApproval([FromBody] KnowledgePagedListDto pagedDto)
        {
            var bmList = await _knowledgeService.GetChildBMId(_sessionContext.RequiredUserId);
            if (bmList == null || bmList.Count == 0)
                return new PagedDto<KnowledgeDataDto>(0, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(new List<Knowledge>()));
            var (total, items) = await _knowledgeRepository
              .Queryable()
              .Includes(it => it.User)
              .Includes(it => it.SystemOrganize)
              .Includes(it => it.KnowledgeType)
              .Where(p => p.Status == EKnowledgeStatus.Auditing && p.IsDeleted == false)
              .Where(p => bmList.Contains(p.CreatorOrgId))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateUserName), d => d.User.Name.Contains(pagedDto.CreateUserName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateBMName), d => d.SystemOrganize.OrgName.Contains(pagedDto.CreateBMName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), d => d.Title.Contains(pagedDto.Title!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.KnowledgeTypeId), d => d.KnowledgeTypeId == pagedDto.KnowledgeTypeId)
              .WhereIF(!string.IsNullOrEmpty(pagedDto.HotspotId), d => d.HotspotId == pagedDto.HotspotId)
              .WhereIF(pagedDto.StartTime != null, d => d.CreationTime >= pagedDto.StartTime)
              .WhereIF(pagedDto.EndTime != null, d => d.CreationTime <= pagedDto.EndTime)
              .OrderByDescending(p => p.CreationTime)
              .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize);

            return new PagedDto<KnowledgeDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(items));
        }

        /// <summary>
        /// 审批完成
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-approved")]
        public async Task<PagedDto<KnowledgeDataDto>> Approved([FromBody] KnowledgePagedListDto pagedDto)
        {
            var bmList = await _knowledgeService.GetChildBMId(_sessionContext.RequiredUserId);
            if (bmList == null || bmList.Count == 0)
                return new PagedDto<KnowledgeDataDto>(0, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(new List<Knowledge>()));
            var (total, items) = await _knowledgeRepository
              .Queryable()
              .Includes(it => it.User)
              .Includes(it => it.SystemOrganize)
              .Includes(it => it.KnowledgeType)
              .Where(p => p.Status == EKnowledgeStatus.Succeed && p.IsDeleted == false)
              .Where(p => bmList.Contains(p.CreatorOrgId))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateUserName), d => d.User.Name.Contains(pagedDto.CreateUserName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateBMName), d => d.SystemOrganize.OrgName.Contains(pagedDto.CreateBMName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), d => d.Title.Contains(pagedDto.Title!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.KnowledgeTypeId), d => d.KnowledgeTypeId == pagedDto.KnowledgeTypeId)
              .WhereIF(!string.IsNullOrEmpty(pagedDto.HotspotId), d => d.HotspotId == pagedDto.HotspotId)
              .WhereIF(pagedDto.StartTime != null, d => d.CreationTime >= pagedDto.StartTime)
              .WhereIF(pagedDto.EndTime != null, d => d.CreationTime <= pagedDto.EndTime)
              .OrderByDescending(p => p.CreationTime)
              .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize);

            return new PagedDto<KnowledgeDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(items));
        }

        /// <summary>
        /// 审批不通过
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-approvalfailed")]
        public async Task<PagedDto<KnowledgeDataDto>> ApprovalFailed([FromBody] KnowledgePagedListDto pagedDto)
        {
            var bmList = await _knowledgeService.GetChildBMId(_sessionContext.UserId);
            if (bmList == null || bmList.Count == 0)
                return new PagedDto<KnowledgeDataDto>(0, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(new List<Knowledge>()));
            var (total, items) = await _knowledgeRepository
              .Queryable()
              .Includes(it => it.User)
              .Includes(it => it.SystemOrganize)
              .Includes(it => it.KnowledgeType)
              .Where(p => p.Status == EKnowledgeStatus.Failed && p.IsDeleted == false)
              .Where(p => bmList.Contains(p.CreatorOrgId))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateUserName), d => d.User.Name.Contains(pagedDto.CreateUserName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateBMName), d => d.SystemOrganize.OrgName.Contains(pagedDto.CreateBMName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), d => d.Title.Contains(pagedDto.Title!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.KnowledgeTypeId), d => d.KnowledgeTypeId == pagedDto.KnowledgeTypeId)
              .WhereIF(!string.IsNullOrEmpty(pagedDto.HotspotId), d => d.HotspotId == pagedDto.HotspotId)
              .WhereIF(pagedDto.StartTime != null, d => d.CreationTime >= pagedDto.StartTime)
              .WhereIF(pagedDto.EndTime != null, d => d.CreationTime <= pagedDto.EndTime)
              .OrderByDescending(p => p.CreationTime)
              .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize);

            return new PagedDto<KnowledgeDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(items));
        }

        #endregion

        #region 知识检索
        /// <summary>
        /// 知识检索
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-knowretrieval")]
        public async Task<PagedDto<KnowledgeRetrievalDataDto>> KnowRetrieval([FromBody] KnowledgeRetrievalPagedListDto pagedDto)
        {
            if (string.IsNullOrEmpty(pagedDto.Keyword))
                throw UserFriendlyException.SameMessage("请输入关键字");

            //是否需要验证本部门以及下级部门的数据，不要注释下面这3行
            var bmList = await _knowledgeService.GetChildBMId(_sessionContext.RequiredUserId);
            if (bmList == null || bmList.Count == 0)
                return new PagedDto<KnowledgeRetrievalDataDto>(0, _mapper.Map<IReadOnlyList<KnowledgeRetrievalDataDto>>(new List<Knowledge>()));

            var (total, items) = await _knowledgeRepository
              .Queryable()
              .Where(p => p.IsDeleted == false && p.Status == EKnowledgeStatus.Succeed)
              .Where(p => bmList.Contains(p.CreatorOrgId)) //是否需要验证本部门以及下级部门的数据，不要注释此行
              .WhereIF(pagedDto.RetrievalType == EKnowledgeRetrievalType.All.ToString(), d => d.Title.Contains(pagedDto.Keyword) || d.Content.Contains(pagedDto.Keyword) || d.Additions.Contains(pagedDto.Keyword))
              .WhereIF(pagedDto.RetrievalType == EKnowledgeRetrievalType.Title.ToString(), d => d.Title.Contains(pagedDto.Keyword))
              .WhereIF(pagedDto.RetrievalType == EKnowledgeRetrievalType.Content.ToString(), d => d.Content.Contains(pagedDto.Keyword))
              .WhereIF(pagedDto.RetrievalType == EKnowledgeRetrievalType.Enclosure.ToString(), d => d.Additions.Contains(pagedDto.Keyword))
              .OrderByDescending(p => p.PageView)
              .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize);

            return new PagedDto<KnowledgeRetrievalDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeRetrievalDataDto>>(items));
        }
        #endregion

        /// <summary>
        /// 待审核
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-tobereviewed")]
        public async Task<PagedDto<KnowledgeApprovalDataDto>> ToBeReviewed([FromBody] KnowledgeApprovalPagedListDto pagedDto)
        {
            var (total, items) = await _knowledgeRepository
              .Queryable()
              .Includes(it => it.User)
              .Includes(it => it.SystemOrganize)
              .Includes(it => it.KnowledgeType)
              .Where(p => p.IsDeleted == false && p.Status == EKnowledgeStatus.Auditing && p.AssignUserIds.Contains(_sessionContext.UserId))//&& p.AssignUserIds.Contains(pagedDto.UserId!)
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateUserName), d => d.User.Name.Contains(pagedDto.CreateUserName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateBMName), d => d.SystemOrganize.OrgName.Contains(pagedDto.CreateBMName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), d => d.Title.Contains(pagedDto.Title!))

              .OrderByDescending(p => p.CreationTime)
              .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize);

            return new PagedDto<KnowledgeApprovalDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeApprovalDataDto>>(items));
        }

        /// <summary>
        /// 审核通过
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-approvedlist")]
        public async Task<PagedDto<KnowledgeApprovalDataDto>> ApprovedList([FromBody] KnowledgeApprovalPagedListDto pagedDto)
        {
            var (total, items) = await _knowledgeRepository
              .Queryable()
              .Includes(it => it.User)
              .Includes(it => it.SystemOrganize)
              .Includes(it => it.KnowledgeType)
              .Where(p => p.IsDeleted == false && p.Status == EKnowledgeStatus.Succeed)//&& p.AssignUserIds.Contains(pagedDto.UserId!)
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateUserName), d => d.User.Name.Contains(pagedDto.CreateUserName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateBMName), d => d.SystemOrganize.OrgName.Contains(pagedDto.CreateBMName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), d => d.Title.Contains(pagedDto.Title!))

              .OrderByDescending(p => p.CreationTime)
              .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize);

            return new PagedDto<KnowledgeApprovalDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeApprovalDataDto>>(items));
        }

        /// <summary>
        /// 审核不通过
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpPost("knowledge-approvedfailedlist")]
        public async Task<PagedDto<KnowledgeApprovalDataDto>> ApprovedFailedList([FromBody] KnowledgeApprovalPagedListDto pagedDto)
        {
            var (total, items) = await _knowledgeRepository
              .Queryable()
              .Includes(it => it.User)
              .Includes(it => it.SystemOrganize)
              .Includes(it => it.KnowledgeType)
              .Where(p => p.IsDeleted == false && p.Status == EKnowledgeStatus.Failed)//&& p.AssignUserIds.Contains(pagedDto.UserId!)
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateUserName), d => d.User.Name.Contains(pagedDto.CreateUserName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.CreateBMName), d => d.SystemOrganize.OrgName.Contains(pagedDto.CreateBMName!))
              .WhereIF(!string.IsNullOrEmpty(pagedDto.Title), d => d.Title.Contains(pagedDto.Title!))

              .OrderByDescending(p => p.CreationTime)
              .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize);

            return new PagedDto<KnowledgeApprovalDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeApprovalDataDto>>(items));
        }

    }
}
